var __commonJS = (cb, mod) => () => (mod || cb((mod = { exports: {} }).exports, mod), mod.exports);
var __esm = (fn, res) => () => (fn && (res = fn(fn = 0)), res);

// src/Constants/index.ts
var TREE_ACTION_TRANSFER, DEFAULT_ROOT;
var init_Constants = __esm(() => {
  (function(TREE_ACTION_TRANSFER2) {
    TREE_ACTION_TRANSFER2["ID"] = "id";
    TREE_ACTION_TRANSFER2["PATH"] = "path";
    TREE_ACTION_TRANSFER2["NODE"] = "node";
  })(TREE_ACTION_TRANSFER || (TREE_ACTION_TRANSFER = {}));
  (function(DEFAULT_ROOT2) {
    DEFAULT_ROOT2["ID"] = "__root__";
    DEFAULT_ROOT2["NAME"] = "__root__";
    DEFAULT_ROOT2[DEFAULT_ROOT2["LEVEL"] = 0] = "LEVEL";
  })(DEFAULT_ROOT || (DEFAULT_ROOT = {}));
});

// src/Utilities/index.ts
var findDeepNode, findNodeByPath, uuid;
var init_Utilities = __esm(() => {
  findDeepNode = (id, treeNode) => {
    for (const item of treeNode) {
      if (item.id === id)
        return item;
      if (item?.children?.length) {
        const foundItem = findDeepNode(id, item.children);
        if (foundItem)
          return foundItem;
      }
    }
    return;
  };
  findNodeByPath = (treeNode, path) => {
    return;
  };
  uuid = () => {
    return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function(c) {
      const r = Math.random() * 16 | 0, v = c == "x" ? r : r & 3 | 8;
      return v.toString(16);
    });
  };
});

// src/Components/TreeNode/index.ts
class HierarchicalTree {
  id = "";
  name = "";
  parent = undefined;
  parentId = "";
  children = [];
  level = undefined;
  root = undefined;
  constructor(props) {
    this.init(props);
  }
  setRoot(node) {
    this.root = node;
    return this;
  }
  init(props) {
    this.id = props.id;
    this.name = props.name ?? "";
    this.parentId = props.parentId;
    this.parent = props.parent;
    if (typeof props.root === "number")
      this.root = props.root;
    this.children = props.children ?? [];
    this.level = props.level;
    return this;
  }
  setLevel(val) {
    this.level = val;
    return this;
  }
  setParent(data) {
    if (data)
      this.parent = data;
    return this;
  }
  findDeep(nodeId) {
    const matched = findDeepNode(nodeId, this.children);
    return matched;
  }
  move(params) {
    const { from, to, type = TREE_ACTION_TRANSFER.NODE } = params;
    let index;
    let fromNode;
    let toNode;
    if (!from)
      fromNode = this;
    switch (type) {
      case TREE_ACTION_TRANSFER.NODE:
        if (from)
          fromNode = from;
        if (to)
          toNode = to;
        break;
      case TREE_ACTION_TRANSFER.ID:
        if (from && typeof from === "string")
          fromNode = this.root?.findDeep(from);
        if (to && typeof to === "string")
          toNode = this.root?.findDeep(to);
        break;
      case TREE_ACTION_TRANSFER.PATH:
        break;
      default:
        break;
    }
    if (!fromNode || !toNode)
      return index;
    if (toNode instanceof HierarchicalTree && fromNode instanceof HierarchicalTree) {
      index = toNode.appendChild([fromNode]);
      fromNode.parent?.removeChild(fromNode.id);
    }
    return index;
  }
  swap(params) {
    const { from, to, type = TREE_ACTION_TRANSFER.NODE } = params;
    let index;
    let fromNode;
    let toNode;
    if (!from)
      fromNode = this;
    switch (type) {
      case TREE_ACTION_TRANSFER.NODE:
        if (from)
          fromNode = from;
        if (to)
          toNode = to;
        break;
      case TREE_ACTION_TRANSFER.ID:
        if (from && typeof from === "string")
          fromNode = this.root?.findDeep(from);
        if (to && typeof to === "string")
          toNode = this.root?.findDeep(to);
        break;
      case TREE_ACTION_TRANSFER.PATH:
        break;
      default:
        break;
    }
    if (!fromNode || !toNode)
      return index;
    if (toNode instanceof HierarchicalTree && fromNode instanceof HierarchicalTree) {
      const cloneToNode = JSON.parse(JSON.stringify(toNode));
      const cloneFromNode = JSON.parse(JSON.stringify(fromNode));
      const parentOfToNode = toNode.parent;
      const parentFromNode = fromNode.parent;
      if (!parentFromNode || !parentOfToNode)
        return index;
      const toNodeInd = parentOfToNode.children.findIndex((item) => item.id === cloneToNode.id);
      const fromNodeInd = parentOfToNode.children.findIndex((item) => item.id === cloneFromNode.id);
      if (toNodeInd === -1 || fromNodeInd === -1)
        return index;
      parentOfToNode.children[toNodeInd] = cloneFromNode;
      parentFromNode.children[fromNodeInd] = cloneToNode;
    }
    return index;
  }
  getRelativePath() {
    const parent = this.parent;
    let path = "";
    if (!parent)
      return DEFAULT_ROOT.ID;
    const index = parent.children.findIndex((item) => item.id === this.id);
    if (index !== -1)
      path = `${parent.getRelativePath()}.children[${index}]`;
    return path;
  }
  removeChild(id) {
    let index = this.children.findIndex((item) => item.id === id);
    if (index !== -1)
      this.children.splice(index, 1);
    return index !== -1 ? index : undefined;
  }
  appendChild(children, place) {
    let index = place;
    if (!children?.length)
      return index;
    if (index === undefined) {
      index = this.children.push(...children);
    } else {
      this.children.splice(index, 0, ...children);
    }
    return;
  }
  toJSON() {
    const getJSONData = (tree) => {
      const jsonData = {
        id: tree.id,
        name: tree.name,
        parentId: tree.parentId,
        level: tree.level,
        children: []
      };
      if (tree.children.length)
        jsonData.children = tree.children.map(getJSONData);
      return jsonData;
    };
    return getJSONData(this);
  }
}
var TreeNode_default;
var init_TreeNode = __esm(() => {
  init_Constants();
  init_Utilities();
  TreeNode_default = HierarchicalTree;
});

// src/Components/Factory/index.ts
class TreeFactory {
  produce(data, rootInfo) {
    const rootTree = this.initRoot(rootInfo);
    const cloneData = JSON.parse(JSON.stringify(data));
    const defaultAcc = { children: [] };
    const converted = cloneData.reduce((accumulator, item, i) => {
      if (!item.parentId || item.parentId === rootTree.id) {
        const treeNode = new TreeNode_default({
          ...item,
          level: 1,
          parentId: rootTree.id,
          parent: rootTree
        }).setRoot(rootTree);
        cloneData[i] = treeNode;
        accumulator.children.push(treeNode);
      } else {
        const matchIndex = cloneData.findIndex((arrItem) => arrItem.id === item.parentId);
        let parentMatch = matchIndex !== -1 ? cloneData[matchIndex] : undefined;
        if (!parentMatch)
          return accumulator;
        if (!Boolean(parentMatch instanceof TreeNode_default)) {
          cloneData[matchIndex] = new TreeNode_default({
            id: parentMatch.id,
            name: parentMatch.name,
            parentId: parentMatch?.parent?.parentId ?? rootTree.id,
            parent: !parentMatch?.parent?.parentId ? rootTree : parentMatch.parent,
            level: !parentMatch?.parent?.parentId ? 1 : Number(parentMatch?.parent?.level) + 1
          }).setRoot(rootTree);
        }
        const treeNode = new TreeNode_default({ ...item, level: Number(parentMatch.level) + 1 }).setRoot(rootTree);
        cloneData[i] = treeNode;
        cloneData[matchIndex].appendChild([treeNode]);
      }
      return accumulator;
    }, defaultAcc);
    rootTree.appendChild(converted.children);
    return rootTree;
  }
  initRoot(rootInfo) {
    let info = {
      id: DEFAULT_ROOT.ID,
      name: DEFAULT_ROOT.NAME,
      level: 0
    };
    if (rootInfo)
      info = {
        ...info,
        ...rootInfo
      };
    const rootTree = new TreeNode_default(info);
    rootTree.setRoot(rootTree);
    return rootTree;
  }
}
var Factory_default;
var init_Factory = __esm(() => {
  init_TreeNode();
  init_Constants();
  Factory_default = TreeFactory;
});

// src/index.ts
var require_src = __commonJS((exports, module) => {
  init_Factory();
  init_TreeNode();
  init_Constants();
  module.exports = {
    TreeFactory: Factory_default,
    HierarchicalTree: TreeNode_default,
    constants: { TREE_ACTION_TRANSFER, DEFAULT_ROOT }
  };
});
export default require_src();
